pice <- read.table("//ad.uni-hamburg.de/basis/mit/bm01/BAB5975/Documents/Lehre/ZRA/Anwendungen/pice/PersDisIncConsExp.txt", header= T)
inc <- pice[,2] #income
con <- pice[,3] #consumption expenditure
ts.plot(inc)
lines(con, col = 'red')

c <- diff(con,1)
c <- c - mean(c)
i <- diff(inc,1)
i <- i - mean(i)

#*******************************************************************************************
#I. Identifikation und vorlufige Schtzung der Transferfunktion 
#I.1 Anpassung eines ARMA-Modells an die Reihe i
par(mfrow = c(1,2))
acf(i, lag.max = 20, xlab=expression(tau), main = "ACF mit Bartlett-Grenzen", ci.type = "ma")
pacf(i, lag.max = 20, xlab=expression(tau), main = "PACF mit White-Noise-Grenzen")
#Entscheidung: Subset-AR[10]-Modells mit 3 Parametern
#s. die Datei Modellanpassung_inc.R

#Anpassung eines Subset-AR[10]-Modells mit 3 Parametern an die Reihe i
fix = c( NA, 0, 0, NA, 0, 0, 0, 0, 0, NA)
out <- arima(i, order = c(10,0,0), fixed =fix, include.mean = F)
out
# ACF und PACF der Residuen
par(mfrow = c(1,2))
acf(out$resid, lag.max = 20, xlab=expression(tau), main = "ACF mit Bartlett-Grenzen", ci.type = "ma")
pacf(out$resid, lag.max = 20, xlab=expression(tau), main = "PACF mit White-Noise-Grenzen")

#Box-Pierce-Ljung Test
res <- out$resid
n.par <- 3 # Anzahl geschtzter Parameter
Box <- matrix(0,4,3)
m <- c(12,14,16,18)
m <- m[m>n.par]
for(j in c(1:length(m))){
  bt <- Box.test(res, lag = m[j], type = "Ljung-Box")
  pw <- 1-pchisq(bt$statistic, m[j]-n.par)
  Box[j,]<- c(bt$parameter, bt$statistic, pw)}
colnames(Box)<- c("Lags", "Statistik", "P-Wert")
Box

#berprfung der Parametersignifikanz
coef <- out$coef[c(1,4,10)]
tstat <- round(coef/diag(out$var.coef)^.5, 4)
pwert<- round(2*(1-pnorm(abs(tstat))),4)
cbind(coef, tstat, pwert)

#I.2 Filtern der Input-Reihe (Prewhitening)
input <- out$resid

#I.3 Filtern der Output-Reihe
fix <- out$coef
out_c <- arima(c, order= c(10,0,0), fixed = fix, include.mean = F)
###Die Residuen entsprechen der gefilterten Output-Reihe
output <- out_c$resid

#I.d Schtzung der Transferfunktion auf der Grundlage der CCF der gefilterten Reihen
par(mfrow = c(1,1))
ccf(input, output, 10) ###Lags -10, -4 und 0
nu <- ccf(input, output, 10)
nu <- c(nu$acf[1], nu$acf[7], nu$acf[11]) * sd(output) / sd(input)

#********************************************************************************
#II. Anpassung eines ARMA-Modells an die Residuenreihe
N <- length(c)
noise <- c[11:N] - i[11:N]*nu[3] - i[7:(N-4)]*nu[2] - i[1:(N-10)]*nu[1]
# ACF und PACF der Reihe noise
par(mfrow = c(1,2))
acf(noise, lag.max = 20, xlab=expression(tau), main = "ACF mit Bartlett-Grenzen", ci.type = "ma")
pacf(noise, lag.max = 20, xlab=expression(tau), main = "PACF mit White-Noise-Grenzen")
#Entscheidung fr AR[5]
#s. Datei Modellanpassung_noise.R

#Anpassung eines Subset-AR[5]-Modells mit 1 Parameter an die Residuenreihe noise
fix = c( 0, 0, 0, 0, NA)
out2 <- arima(noise, order = c(5,0,0), fixed =fix, include.mean = F)
out2
# ACF und PACF der neuen Residuen
par(mfrow = c(1,2))
acf(out2$resid, lag.max = 20, xlab=expression(tau), main = "ACF mit Bartlett-Grenzen", ci.type = "ma")
pacf(out2$resid, lag.max = 20, xlab=expression(tau), main = "PACF mit White-Noise-Grenzen")

#berprfung der Parametersignifikanz
coef <- out2$coef[c(5)]
tstat <- round(coef/diag(out2$var.coef)^.5, 4)
pwert<- round(2*(1-pnorm(abs(tstat))),4)
cbind(coef, tstat, pwert)

#********************************************************************************
#III. Gesamtschtzung des Transferfunktionsmodells
trans <- cbind(i[11:N],i[7:(N-4)],i[1:(N-10)])
fix = c( 0, 0, 0, 0, NA, NA, NA, NA)
out <- arima(c[11:N], order = c(5,0,0), fixed =fix, xreg = trans, include.mean = F)
out

#*******************************************************************************************
#Die geschtzte Modell-Gleichung:
#c = (0.6171 + 0.2945 B^4 + 0.2133 B^10)i + noise
#noise = -0.5226 B^5 noise + epsilon
#c = (1-B)con - m_c, m_c = 141.0154 ist der Mittelwert der Reihe (1-B)con 
#i = (1-B)inc - m_i, m_i = 152.1385 ist der Mittelwert der Reihe (1-B)inc